WRONG_CONFIG_CHANNELS_SHOULD_BE_LIST_TUPLE = (
    'Wrong configuration, channels is a list or tuple of channel names.')
WRONG_CONFIG_CHANNEL_NAME_SHOULD_BE_STR = 'Wrong configuration, channel names should be a string.'
